import _ from "./lodash.js";
import Graph from "./graph.js";

const writeNodes = g => _.map(g.nodes(), v => {
    const nodeValue = g.node(v);
    const parent = g.parent(v);
    const node = { v: v };
    if (!_.isUndefined(nodeValue)) {
        node.value = nodeValue;
    }
    if (!_.isUndefined(parent)) {
        node.parent = parent;
    }
    return node;
});

const writeEdges = g => _.map(g.edges(), e => {
    const edgeValue = g.edge(e);
    const edge = { v: e.v, w: e.w };
    if (!_.isUndefined(e.name)) {
        edge.name = e.name;
    }
    if (!_.isUndefined(edgeValue)) {
        edge.value = edgeValue;
    }
    return edge;
});

const write = g => {
    const json = {
        options: {
            directed: g.isDirected(),
            multigraph: g.isMultigraph(),
            compound: g.isCompound()
        },
        nodes: writeNodes(g),
        edges: writeEdges(g)
    };
    if (!_.isUndefined(g.graph())) {
        json.value = _.clone(g.graph());
    }
    return json;
};


const read = json => {
    const g = new Graph(json.options).setGraph(json.value);
    _.each(json.nodes, entry => {
        g.setNode(entry.v, entry.value);
        if (entry.parent) {
            g.setParent(entry.v, entry.parent);
        }
    });
    _.each(json.edges, entry => g.setEdge({ v: entry.v, w: entry.w, name: entry.name }, entry.value));
    return g;
};

export default {
    write,
    read
};
